<?php

namespace IQBall\Api;

use Exception;
use IQBall\Core\Action;
use IQBall\Core\Connection;
use IQBall\Core\Data\Account;
use IQBall\Core\Gateway\AccountGateway;
use IQBall\Core\Http\HttpResponse;
use IQBall\Core\Http\JsonHttpResponse;
use IQBall\Core\Session\PhpSessionHandle;
use IQBall\Core\Validation\ValidationFail;

class API {
    public static function render(HttpResponse $response): void {
        http_response_code($response->getCode());

        foreach ($response->getHeaders() as $header => $value) {
            header("$header: $value");
        }

        if ($response instanceof JsonHttpResponse) {
            header('Content-type: application/json');
            echo $response->getJson();
        } else {
            throw new Exception("API returned a non-json response.");
        }
    }

    /**
     * @param mixed[] $match
     * @return HttpResponse
     * @throws Exception
     */
    public static function handleMatch(array $match): HttpResponse {
        if (!$match) {
            return new JsonHttpResponse([ValidationFail::notFound("not found")]);
        }

        $action = $match['target'];
        if (!$action instanceof Action) {
            throw new Exception("routed action is not an AppAction object.");
        }

        $auth = null;

        if ($action->isAuthRequired()) {
            $auth = self::tryGetAuthorization();
            if ($auth == null) {
                return new JsonHttpResponse([ValidationFail::unauthorized("Missing or invalid 'Authorization' header.")]);
            }
        }

        return $action->run($match['params'], $auth);
    }

    private static function tryGetAuthorization(): ?Account {
        $headers = getallheaders();

        // If no authorization header is set, try fallback to php session.
        if (!isset($headers['Authorization'])) {
            $session = PhpSessionHandle::init();
            return $session->getAccount();
        }

        $token = $headers['Authorization'];
        $gateway = new AccountGateway(new Connection(get_database()));
        return $gateway->getAccountFromToken($token);
    }
}
