<?php

namespace IQBall\Api\Controller;

use IQBall\Core\Route\Control;
use IQBall\Core\Data\Account;
use IQBall\Core\Http\HttpCodes;
use IQBall\Core\Http\HttpRequest;
use IQBall\Core\Http\HttpResponse;
use IQBall\Core\Http\JsonHttpResponse;
use IQBall\Core\Model\TacticModel;
use IQBall\Core\Validation\Validators;

/**
 * API endpoint related to tactics
 */
class APITacticController {
    private TacticModel $model;

    /**
     * @param TacticModel $model
     */
    public function __construct(TacticModel $model) {
        $this->model = $model;
    }

    /**
     * update name of tactic, specified by tactic identifier, given in url.
     * @param int $tactic_id
     * @param Account $account
     * @return HttpResponse
     */
    public function updateName(int $tactic_id, Account $account): HttpResponse {
        return Control::runChecked([
            "name" => [Validators::lenBetween(1, 50), Validators::nameWithSpaces()],
        ], function (HttpRequest $request) use ($tactic_id, $account) {

            $failures = $this->model->updateName($tactic_id, $request["name"], $account->getId());

            if (!empty($failures)) {
                //TODO find a system to handle Unauthorized error codes more easily from failures.
                return new JsonHttpResponse($failures, HttpCodes::BAD_REQUEST);
            }

            return HttpResponse::fromCode(HttpCodes::OK);
        }, true);
    }
}
