<?php

namespace IQBall\App\Controller;

use IQBall\Core\Http\HttpCodes;
use IQBall\Core\Http\HttpResponse;
use IQBall\App\ViewHttpResponse;
use IQBall\Core\Model\TacticModel;
use IQBall\Core\Session\SessionHandle;
use IQBall\Core\Validator\TacticValidator;

class VisualizerController {
    private TacticModel $tacticModel;

    /**
     * @param TacticModel $tacticModel
     */
    public function __construct(TacticModel $tacticModel) {
        $this->tacticModel = $tacticModel;
    }

    /**
     * opens a visualisation page for the tactic specified by its identifier in the url.
     * @param int $id
     * @param SessionHandle $session
     * @return HttpResponse
     */
    public function openVisualizer(int $id, SessionHandle $session): HttpResponse {
        $tactic = $this->tacticModel->get($id);

        $failure = TacticValidator::validateAccess($tactic, $id, $session->getAccount()->getId());

        if ($failure != null) {
            return ViewHttpResponse::twig('error.html.twig', ['failures' => [$failure]], HttpCodes::NOT_FOUND);
        }

        return ViewHttpResponse::react("views/Visualizer.tsx", ["name" => $tactic->getName()]);
    }
}
